@@ -8,21 +8,162 @@ snspocket: the container class for snsapi's
 # === system imports ===
 from utils import json
 from os import path
+import sqlite3
+import thread
 
 # === snsapi modules ===
 import snstype
 import utils
 from errors import snserror
-from utils import console_output
+from utils import console_output, obj2str, str2obj
 from snslog import SNSLog as logger
 from snsconf import SNSConf
 import platform
+from async import AsyncDaemonWithCallBack
 
 # === 3rd party modules ===
 
 DIR_DEFAULT_CONF_CHANNEL = path.join(SNSConf.SNSAPI_DIR_STORAGE_CONF, 'channel.json')
 DIR_DEFAULT_CONF_POCKET = path.join(SNSConf.SNSAPI_DIR_STORAGE_CONF, 'pocket.json')
 
+
+def _default_callback(pocket, res):
+    pass
+
+class BackgroundOperationPocketWithSQLite:
+    def __init__(self, pocket, sqlite, callback=_default_callback, timeline_sleep=60, update_sleep=10):
+        self.sp = pocket
+        self.dblock = thread.allocate_lock()
+        self.sqlitefile = sqlite
+        conn = sqlite3.connect(self.sqlitefile)
+        c = conn.cursor()
+        c.execute("""CREATE TABLE IF NOT EXISTS home_timeline (
+            id integer primary key, pickled_object text, digest text, text text, username text, userid text, time integer, isread integer DEFAULT 0
+        )""")
+        c.execute("""CREATE TABLE IF NOT EXISTS pending_update (
+            id integer primary key, callback text, type text, args text, kwargs text
+        )""")
+        conn.commit()
+        c.close()
+        self.home_timeline_job = AsyncDaemonWithCallBack(self.sp.home_timeline, (), {}, self.write_timeline_to_db, timeline_sleep)
+        self.update_job = AsyncDaemonWithCallBack(self.update_func, (), {}, None, update_sleep)
+        self.home_timeline_job.start()
+        self.update_job.start()
+
+    def home_timeline(self, count=20):
+        ret = snstype.MessageList()
+        logger.debug("acquiring lock")
+        self.dblock.acquire()
+        try:
+            conn = sqlite3.connect(self.sqlitefile)
+            c = conn.cursor()
+            c.execute("SELECT pickled_object FROM home_timeline ORDER BY time DESC LIMIT 0, %d" % (count,))
+            p = c.fetchall()
+            logger.info("%d messages read from database" % (len(p)))
+            for i in p:
+                ret.append(str2obj(str(i[0])))
+        except Exception, e:
+            logger.warning("Error while reading database: %s" % (str(e)))
+        finally:
+            logger.debug("releasing lock")
+            self.dblock.release()
+            return ret
+
+    def write_timeline_to_db(self, msglist):
+        logger.debug("acquiring lock")
+        self.dblock.acquire()
+        try:
+            conn = sqlite3.connect(self.sqlitefile)
+            cursor = conn.cursor()
+            what_to_write = [
+            ]
+            for msg in msglist:
+                try:
+                    pickled_msg = obj2str(msg)
+                    sig = unicode(msg.digest())
+                    cursor.execute("SELECT * FROM home_timeline WHERE digest = ?", (sig,))
+                    if not cursor.fetchone():
+                        what_to_write.append((
+                            unicode(pickled_msg), sig, msg.parsed.text, msg.parsed.username, msg.parsed.userid, msg.parsed.time
+                        ))
+                except Exception, e:
+                    logger.warning("Error while checking message: %s" % (str(e)))
+            try:
+                logger.info("Writing %d messages" % (len(what_to_write)))
+                cursor.executemany("INSERT INTO home_timeline (pickled_object, digest, text, username, userid, time) VALUES(?, ?, ?, ?, ?, ?)", what_to_write)
+            except Exception, e:
+                logger.warning("Error %s" % (str(e)))
+            conn.commit()
+            cursor.close()
+        finally:
+            logger.debug("releasing lock")
+            self.dblock.release()
+
+    def _update(self, type, args, kwargs):
+        logger.debug("acquiring lock")
+        self.dblock.acquire()
+        try:
+            conn = sqlite3.connect(self.sqlitefile)
+            cursor = conn.cursor()
+            callback = None
+            if 'callback' in kwargs:
+                callback = kwargs['callback']
+                del kwargs['callback']
+            cursor.execute("INSERT INTO pending_update (type, callback, args, kwargs) VALUES (?, ?, ?, ?)", (
+                type,
+                obj2str(callback),
+                obj2str(args),
+                obj2str(kwargs)
+            ))
+            conn.commit()
+            cursor.close()
+            return True
+        except Exception, e:
+            logger.warning("Error while saving pending_update: %s" % (str(e)))
+            return False
+        finally:
+            logger.debug("releasing lock")
+            self.dblock.release()
+
+    def update(self, *args, **kwargs):
+        return self._update('update', args, kwargs)
+
+    def forward(self, *args, **kwargs):
+        return self._update('forward', args, kwargs)
+
+    def reply(self, *args, **kwargs):
+        return self._update('reply', args, kwargs)
+
+    def update_func(self):
+        logger.debug("acquiring lock")
+        self.dblock.acquire()
+        try:
+            conn = sqlite3.connect(self.sqlitefile)
+            conn.row_factory = sqlite3.Row
+            cursor = conn.cursor()
+            cursor.execute("SELECT * FROM pending_update")
+            i = cursor.fetchone()
+            if i:
+                cursor.execute("DELETE FROM pending_update WHERE id = ?", (i['id'], ))
+                j = {
+                    'id': str(i['id']),
+                    'args': str2obj(str(i['args'])),
+                    'kwargs': str2obj(str(i['kwargs'])),
+                    'type': str(i['type']),
+                    'callback': str2obj(str(i['callback']))
+                }
+                res = getattr(self.sp, j['type'])(*j['args'], **j['kwargs'])
+                if j['callback']:
+                    j['callback'](self, res)
+            conn.commit()
+            cursor.close()
+        except Exception, e:
+            logger.warning("Error while updating: %s" % (str(e)))
+        finally:
+            logger.debug("releasing lock")
+            self.dblock.release()
+
+
 class SNSPocket(dict):
     """The container class for snsapi's"""
 
@@ -286,7 +427,7 @@ class SNSPocket(dict):
         logger.info("Read %d statuses", len(status_list))
         return status_list
 
-    def update(self, text, channel = None):
+    def update(self, text, channel = None, **kwargs):
         """
         Route to update method of snsapi.
 
@@ -305,7 +446,7 @@ class SNSPocket(dict):
         else:
             for c in self.itervalues():
                 if self.__check_method(c, 'update') and not c.is_expired():
-                    re[c.jsonconf['channel_name']] = c.update(text)
+                    re[c.jsonconf['channel_name']] = c.update(text, **kwargs)
 
         logger.info("Update status '%s'. Result:%s", text, re)
         return re
